@@ -48,6 +48,8 @@ has_virtual_mem_fn_change(const function_decl_diff* diff);
 bool
 has_class_decl_only_def_change(const class_or_union_sptr& first,
 			       const class_or_union_sptr& second);
+bool
+has_class_decl_only_def_change(const diff *diff);
 
 bool
 has_basic_type_name_change(const diff *);
@@ -58,6 +60,9 @@ has_class_or_union_type_name_change(const diff *d);
 bool
 has_basic_or_class_type_name_change(const diff *d);
 
+bool
+is_mostly_distinct_diff(const diff *d);
+
 struct filter_base;
 /// Convenience typedef for a shared pointer to filter_base
 typedef shared_ptr<filter_base> filter_base_sptr;
