@@ -80,6 +80,9 @@ struct diff_sptr_hasher
 /// Convenience typedef for a vector of @ref diff_sptr.
 typedef vector<diff_sptr> diff_sptrs_type;
 
+/// Convenience typedef for a vector of @ref diff*.
+typedef vector<diff*> diff_ptrs_type;
+
 /// Convenience typedef for an unoredered set of @ref diff_sptr
 typedef unordered_set<diff_sptr, diff_sptr_hasher> unordered_diff_sptr_set;
 
@@ -672,6 +675,18 @@ public:
   bool
   show_leaf_changes_only() const;
 
+  bool
+  show_hex_values() const;
+
+  void
+  show_hex_values(bool f);
+
+  bool
+  show_offsets_sizes_in_bits() const;
+
+  void
+  show_offsets_sizes_in_bits(bool f);
+
   void
   show_relative_offset_changes(bool f);
 
@@ -2439,6 +2454,27 @@ public:
   void num_leaf_changes_filtered_out(size_t);
 
   size_t net_num_leaf_changes() const;
+
+  size_t num_leaf_type_changes() const;
+  void num_leaf_type_changes(size_t);
+
+  size_t num_leaf_type_changes_filtered_out() const;
+  void num_leaf_type_changes_filtered_out(size_t);
+  size_t net_num_leaf_type_changes() const;
+
+  size_t num_leaf_func_changes() const;
+  void num_leaf_func_changes(size_t);
+
+  size_t num_leaf_func_changes_filtered_out() const;
+  void num_leaf_func_changes_filtered_out(size_t);
+  size_t net_num_leaf_func_changes() const;
+
+  size_t num_leaf_var_changes() const;
+  void num_leaf_var_changes(size_t);
+
+  size_t num_leaf_var_changes_filtered_out() const;
+  void num_leaf_var_changes_filtered_out(size_t);
+  size_t net_num_leaf_var_changes() const;
 }; // end class corpus_diff::diff_stats
 
 /// The base class for the node visitors.  These are the types used to
@@ -2631,6 +2667,12 @@ is_class_diff(const diff* diff);
 const union_diff*
 is_union_diff(const diff* diff);
 
+const class_or_union_diff*
+is_class_or_union_diff(const diff* d);
+
+const class_or_union_diff*
+is_anonymous_class_or_union_diff(const diff* d);
+
 const array_diff*
 is_array_diff(const diff* diff);
 
