@@ -407,8 +407,8 @@ decl_name_changed(const diff *d)
 
 /// Test if two decls represents a harmless name change.
 ///
-/// For now, a harmless name change is a name change for a typedef,
-/// enum or data member.
+/// For now, a harmless name change is considered only for a typedef,
+/// enum or a data member.
 ///
 /// @param f the first decl to consider in the comparison.
 ///
@@ -418,10 +418,27 @@ decl_name_changed(const diff *d)
 bool
 has_harmless_name_change(const decl_base_sptr& f, const decl_base_sptr& s)
 {
+  // So, a harmless name change is either ...
   return (decl_name_changed(f, s)
-	  && ((is_typedef(f) && is_typedef(s))
-	      || (is_data_member(f) && is_data_member(s))
-	      || (is_enum_type(f) && is_enum_type(s))));
+	  && (// ... a typedef name change, without having the
+	      // underlying type changed ...
+	      (is_typedef(f)
+	       && is_typedef(s)
+	       && (is_typedef(f)->get_underlying_type()
+		   == is_typedef(s)->get_underlying_type()))
+	      // .. or a data member name change, without having the
+	      // its type changed ...
+	      || (is_data_member(f)
+		  && is_data_member(s)
+		  && (is_var_decl(f)->get_type()
+		      == is_var_decl(s)->get_type()))
+	      // .. an enum name change without having any other part
+	      // of the enum to change.
+	      || (is_enum_type(f)
+		  && is_enum_type(s)
+		  && !enum_has_non_name_change(*is_enum_type(f),
+					       *is_enum_type(s),
+					       0))));
 }
 
 /// Test if a class_diff node has non-static members added or
@@ -792,7 +809,7 @@ has_class_decl_only_def_change(const class_or_union_sptr& first,
 //// @return true if the class_or_union_diff carries a change in which
 /// the two classes are different by the fact that one is a decl-only
 /// and the other one is defined.
-static bool
+bool
 has_class_decl_only_def_change(const diff *diff)
 {
   const class_or_union_diff *d = dynamic_cast<const class_or_union_diff*>(diff);
@@ -851,6 +868,51 @@ has_basic_or_class_type_name_change(const diff *d)
 	  || has_class_or_union_type_name_change(d));
 }
 
+/// Test if a diff node carries a distinct type change or a
+/// pointer/reference/typedef to distinct type change.
+///
+/// Note that a distinct type change is a change where the two
+/// subjects of the change are not of the same kind, e.g, a basic type
+/// that got changed into a qualified type.
+///
+/// @param d the diff node to consider.
+///
+/// @return true iff @p d is mostly a distinct diff.
+bool
+is_mostly_distinct_diff(const diff *d)
+{
+  if (is_distinct_diff(d))
+    return true;
+
+  // Let's consider that 'd' is a type diff ...
+  type_diff_base* td = const_cast<type_diff_base*>(is_type_diff(d));
+  if (!td)
+    {
+      // ... or a function parameter diff.  In which case, let's get
+      // its child type diff.
+      fn_parm_diff *pd = const_cast<fn_parm_diff*>(is_fn_parm_diff(d));
+      if (pd)
+	td = const_cast<type_diff_base*>(is_type_diff(pd->type_diff().get()));
+      else
+	return false;
+    }
+
+  // At this point, if we are not looking at a type diff we must have
+  // bailed out already.
+  assert(td);
+
+  type_base_sptr first = is_type(td->first_subject());
+  type_base_sptr second = is_type(td->second_subject());
+
+  first = peel_typedef_pointer_or_reference_type(first,
+						 /*peel_qualified_type=*/false);
+  second = peel_typedef_pointer_or_reference_type(second,
+						  /*peel_qual_type=*/false);
+  assert(first && second);
+
+  return distinct_diff::entities_are_of_distinct_kinds(first, second);
+}
+
 /// Test if an enum_diff carries an enumerator insertion.
 ///
 /// @param diff the enum_diff to consider.
