@@ -98,6 +98,10 @@ default_reporter::report(const enum_diff& d, ostream& out,
   if (!d.to_be_reported())
     return;
 
+  RETURN_IF_BEING_REPORTED_OR_WAS_REPORTED_EARLIER3(d.first_subject(),
+						   d.second_subject(),
+						   "enum type");
+
   string name = d.first_enum()->get_pretty_representation();
 
   enum_type_decl_sptr first = d.first_enum(), second = d.second_enum();
@@ -185,6 +189,8 @@ default_reporter::report(const enum_diff& d, ostream& out,
   if (d.context()->show_leaf_changes_only())
     maybe_report_interfaces_impacted_by_diff(&d, out, indent,
 					     /*new_line_prefix=*/false);
+
+  d.reported_once(true);
 }
 
 /// For a @ref typedef_diff node, report the changes that are local.
@@ -210,11 +216,11 @@ default_reporter::report_local_typedef_changes(const typedef_diff &d,
 
   maybe_report_diff_for_member(f, s, d.context(), out, indent);
 
-  if (filtering::has_harmless_name_change(f, s)
-      && ((d.context()->get_allowed_category()
-	   & HARMLESS_DECL_NAME_CHANGE_CATEGORY)
-	  ||
-	  d.context()->show_leaf_changes_only()))
+  if ((filtering::has_harmless_name_change(f, s)
+       && ((d.context()->get_allowed_category()
+	    & HARMLESS_DECL_NAME_CHANGE_CATEGORY)
+	   || d.context()->show_leaf_changes_only()))
+      || f->get_qualified_name() != s->get_qualified_name())
     {
       out << indent << "typedef name changed from "
 	  << f->get_qualified_name()
@@ -245,25 +251,55 @@ default_reporter::report(const typedef_diff& d,
     return;
 
   typedef_decl_sptr f = d.first_typedef_decl(), s = d.second_typedef_decl();
-  RETURN_IF_BEING_REPORTED_OR_WAS_REPORTED_EARLIER(f, s);
 
   bool emit_nl = report_local_typedef_changes(d, out, indent);
 
   diff_sptr dif = d.underlying_type_diff();
-  if (dif && dif->to_be_reported())
+  if (dif && dif->has_changes())
     {
-      RETURN_IF_BEING_REPORTED_OR_WAS_REPORTED_EARLIER2(dif, "underlying type");
-      out << indent
-	  << "underlying type '"
-	  << dif->first_subject()->get_pretty_representation() << "'";
-      report_loc_info(dif->second_subject(), *d.context(), out);
-      out << " changed:\n";
-      dif->report(out, indent + "  ");
-      emit_nl = false;
+      if (dif->to_be_reported())
+	{
+	  RETURN_IF_BEING_REPORTED_OR_WAS_REPORTED_EARLIER2(dif,
+							    "underlying type");
+	  out << indent
+	      << "underlying type '"
+	      << dif->first_subject()->get_pretty_representation() << "'";
+	  report_loc_info(dif->first_subject(), *d.context(), out);
+	  out << " changed:\n";
+	  dif->report(out, indent + "  ");
+	  emit_nl = false;
+	}
+      else
+	{
+	  // The typedef change is to be reported, so we'll report its
+	  // underlying type change too (even if its redundant),
+	  // unless it's suppressed.  It makes sense in this
+	  // particular case to emit the underlying type change
+	  // because of the informative value underneath.  We don't
+	  // want to just know about the local changes of the typedef,
+	  // but also about the changes on the underlying type.
+	  diff_category c = dif->get_category();
+	  if (!(c & SUPPRESSED_CATEGORY))
+	    {
+	      out << indent
+		  << "underlying type '"
+		  << dif->first_subject()->get_pretty_representation() << "'";
+	      report_loc_info(dif->first_subject(), *d.context(), out);
+	      out << " changed:\n";
+	      if (c & REDUNDANT_CATEGORY)
+		dif->set_category(c & ~REDUNDANT_CATEGORY);
+	      dif->report(out, indent + "  ");
+	      if (c & REDUNDANT_CATEGORY)
+		dif->set_category(c | REDUNDANT_CATEGORY);
+	      emit_nl = false;
+	    }
+	}
     }
 
   if (emit_nl)
     out << "\n";
+
+  d.reported_once(true);
 }
 
 /// For a @ref qualified_type_diff node, report the changes that are
@@ -455,7 +491,11 @@ default_reporter::report(const fn_parm_diff& d, ostream& out,
 
   if (d.to_be_reported())
     {
-      assert(d.type_diff() && d.type_diff()->to_be_reported());
+      diff_sptr type_diff = d.type_diff();
+      assert(type_diff->has_changes());
+      diff_category saved_category = type_diff->get_category();
+      // Parameter type changes are never redundants.
+      type_diff->set_category(saved_category & ~REDUNDANT_CATEGORY);
       out << indent
 	  << "parameter " << f->get_index();
       report_loc_info(f, *d.context(), out);
@@ -467,7 +507,8 @@ default_reporter::report(const fn_parm_diff& d, ostream& out,
       else
 	out << "' changed:\n";
 
-      d.type_diff()->report(out, indent + "  ");
+      type_diff->report(out, indent + "  ");
+      type_diff->set_category(saved_category);
     }
 }
 
@@ -517,7 +558,6 @@ default_reporter::report_local_function_type_changes(const function_type_diff& d
   // Hmmh, the above was quick.  Now report about function parameters;
   // this shouldn't be as straightforward.
 
-
   // Report about the parameters that got removed.
   bool emitted = false;
   for (vector<function_decl::parameter_sptr>::const_iterator i =
@@ -1536,7 +1576,7 @@ default_reporter::report(const function_decl_diff& d, ostream& out,
 
   maybe_report_diff_for_symbol(ff->get_symbol(),
 			       sf->get_symbol(),
-			       out, indent);
+			       d.context(), out, indent);
 
   // Now report about inline-ness changes
   if (ff->is_declared_inline() != sf->is_declared_inline())
@@ -1651,7 +1691,7 @@ default_reporter::report(const var_diff& d, ostream& out,
 
   maybe_report_diff_for_symbol(d.first_var()->get_symbol(),
 			       d.second_var()->get_symbol(),
-			       out, indent);
+			       d.context(), out, indent);
 
   maybe_report_diff_for_member(first, second, d.context(), out, indent);
 
