@@ -168,18 +168,14 @@ static bool
 char_is_white_space(int b)
 {return b == ' ' || b == '\t' || b == '\n';}
 
-/// Remove the trailing spaces at the begining and at the end of a
-/// given string.
+/// Remove the spaces at the begining and at the end of a given string.
 ///
-/// @param str the string to remove trailing white spaces from.
+/// @param str the string to remove leading and trailing white spaces from.
 ///
-/// @return the string resulting from the removal of trailing white
-/// space from @p str.
+/// @return the string resulting from the removal of white space from @p str.
 static string
-remove_trailing_white_spaces(const string& str)
+trim_white_space(const string& str)
 {
-  string result;
-
   if (str.empty())
     return str;
 
@@ -193,10 +189,7 @@ remove_trailing_white_spaces(const string& str)
     if (!char_is_white_space(str[e]))
       break;
 
-  for (unsigned i = s; i <= e; ++i)
-    result += str[i];
-
-  return result;
+  return str.substr(s, e - s + 1);
 }
 
 // <property stuff>
@@ -1401,8 +1394,7 @@ public:
 	assert(read_next_char(c));
 	v += c;
       }
-    string result = remove_trailing_white_spaces(v);
-    return result;
+    return trim_white_space(v);
   }
 
   /// Read a string property value.
