@@ -59,29 +59,80 @@ report_diffs(const reporter_base& r,
 	     ostream& out,
 	     const string& indent)
 {
+  diff_ptrs_type sorted_diffs;
+  sort_string_diff_ptr_map(mapp, sorted_diffs);
+
   bool started_to_emit = false;
-  for (string_diff_ptr_map::const_iterator i = mapp.begin();
-       i != mapp.end();
+  for (diff_ptrs_type::const_iterator i = sorted_diffs.begin();
+       i != sorted_diffs.end();
        ++i)
     {
-      if (const var_diff *d = is_var_diff(i->second))
+      if (const var_diff *d = is_var_diff(*i))
 	if (is_data_member(d->first_var()))
 	  continue;
 
-      if (r.diff_to_be_reported(i->second->get_canonical_diff()))
+      if (r.diff_to_be_reported((*i)->get_canonical_diff()))
 	{
 	  if (started_to_emit)
 	    out << "\n\n";
 
-	  string n = i->second->first_subject()->get_pretty_representation();
-	  out << indent << "'" << n << "' changed:\n";
-	  i->second->get_canonical_diff()->report(out, indent + "  ");
+	  string n = (*i)->first_subject()->get_pretty_representation();
+
+	  out << indent << "'" << n ;
+
+	  report_loc_info((*i)->first_subject(),
+			  *(*i)->context(), out);
+
+	  out << "' changed:\n";
+
+	  (*i)->get_canonical_diff()->report(out, indent + "  ");
 	  out << "\n";
 	  started_to_emit = true;
 	}
     }
 }
 
+/// Report the type changes carried by an instance of @ref diff_maps.
+///
+/// @param maps the set of diffs to report.
+///
+/// @param out the output stream to report the diffs to.
+///
+/// @param indent the string to use for indentation.
+static void
+report_type_changes_from_diff_maps(const leaf_reporter& reporter,
+				   const diff_maps& maps,
+				   ostream& out,
+				   const string& indent)
+{
+  // basic types
+  report_diffs(reporter, maps.get_type_decl_diff_map(), out, indent);
+
+  // enums
+  report_diffs(reporter, maps.get_enum_diff_map(), out, indent);
+
+  // classes
+  report_diffs(reporter, maps.get_class_diff_map(), out, indent);
+
+  // unions
+  report_diffs(reporter, maps.get_union_diff_map(), out, indent);
+
+  // typedefs
+  report_diffs(reporter, maps.get_typedef_diff_map(), out, indent);
+
+  // arrays
+  report_diffs(reporter, maps.get_array_diff_map(), out, indent);
+
+  // It doesn't make sense to report function type changes, does it?
+  // report_diffs(reporter, maps.get_function_type_diff_map(), out, indent);
+
+  // distinct diffs
+  report_diffs(reporter, maps.get_distinct_diff_map(), out, indent);
+
+  // function parameter diffs
+  report_diffs(reporter, maps.get_fn_parm_diff_map(), out, indent);
+}
+
 /// Report the changes carried by an instance of @ref diff_maps.
 ///
 /// @param maps the set of diffs to report.
@@ -94,38 +145,13 @@ leaf_reporter::report_changes_from_diff_maps(const diff_maps& maps,
 					     ostream& out,
 					     const string& indent) const
 {
-  // basic types
-  report_diffs(*this, maps.get_type_decl_diff_map(), out, indent);
-
-  // enums
-  report_diffs(*this, maps.get_enum_diff_map(), out, indent);
-
-  // classes
-  report_diffs(*this, maps.get_class_diff_map(), out, indent);
-
-  // unions
-  report_diffs(*this, maps.get_union_diff_map(), out, indent);
-
-  // typedefs
-  report_diffs(*this, maps.get_typedef_diff_map(), out, indent);
-
-  // arrays
-  report_diffs(*this, maps.get_array_diff_map(), out, indent);
-
-  // function types
-  report_diffs(*this, maps.get_function_type_diff_map(), out, indent);
+  report_type_changes_from_diff_maps(*this, maps, out, indent);
 
   // function decls
   report_diffs(*this, maps.get_function_decl_diff_map(), out, indent);
 
   // var decl
   report_diffs(*this, maps.get_var_decl_diff_map(), out, indent);
-
-  // distinct diffs
-  report_diffs(*this, maps.get_distinct_diff_map(), out, indent);
-
-  // function parameter diffs
-  report_diffs(*this, maps.get_fn_parm_diff_map(), out, indent);
 }
 
 /// Report the changes carried by a @ref typedef_diff node.
@@ -166,10 +192,21 @@ leaf_reporter::report(const qualified_type_diff& d, ostream& out,
 /// Note that this function does nothing because a @ref pointer_diff
 /// node never carries local changes.
 void
-leaf_reporter::report(const pointer_diff&, ostream&, const string&) const
+leaf_reporter::report(const pointer_diff &d,
+		      ostream& out,
+		      const string& indent) const
 {
-  // A pointer_diff does not carry local changes.
-  return;
+  // Changes that modify the representation of a pointed-to type is
+  // considered local to the pointer type.
+  if (!diff_to_be_reported(&d))
+    return;
+
+  out << indent
+      << "pointer type changed from: '"
+      << d.first_pointer()->get_pretty_representation()
+      << "' to: '"
+      << d.second_pointer()->get_pretty_representation()
+      << "'";
 }
 
 /// Report the changes carried by a @ref reference_diff node.
@@ -598,18 +635,48 @@ leaf_reporter::report(const class_or_union_diff& d,
 	    }
 	}
 
-      // report change
+      // report changes
       size_t numchanges =
-	d.class_or_union_diff::get_priv()->sorted_subtype_changed_dm_.size();
+	d.class_or_union_diff::get_priv()->sorted_changed_dm_.size();
       size_t num_filtered =
 	d.class_or_union_diff::get_priv()->
-	count_filtered_subtype_changed_dm(/*local_only =*/ true);
+	count_filtered_changed_dm(/*local_only =*/ true);
       assert(numchanges >= num_filtered);
       size_t net_numchanges = numchanges - num_filtered;
 
+      bool emitted_data_members_changes = false;
       if (net_numchanges)
 	{
 	  report_mem_header(out, subtype_change_kind, "data member", indent);
+	  for (var_diff_sptrs_type::const_iterator it =
+		 d.class_or_union_diff::get_priv()->
+		 sorted_changed_dm_.begin();
+	       it != d.class_or_union_diff::get_priv()->
+		 sorted_changed_dm_.end();
+	       ++it)
+	    {
+	      if (diff_to_be_reported((*it).get()))
+		{
+		  represent(*it, ctxt, out, indent + " ",
+			    /*local_only=*/true);
+		  out << "\n";
+		  emitted_data_members_changes = true;
+		}
+	    }
+	}
+
+      numchanges =
+	d.class_or_union_diff::get_priv()->sorted_subtype_changed_dm_.size();
+      num_filtered =
+	d.class_or_union_diff::get_priv()->
+	count_filtered_subtype_changed_dm(/*local_only =*/ true);
+      assert(numchanges >= num_filtered);
+      net_numchanges = numchanges - num_filtered;
+
+      if (net_numchanges)
+	{
+	  if (!emitted_data_members_changes)
+	    report_mem_header(out, change_kind, "data member", indent);
 	  for (var_diff_sptrs_type::const_iterator it =
 		 d.class_or_union_diff::get_priv()->sorted_subtype_changed_dm_.begin();
 	       it != d.class_or_union_diff::get_priv()->sorted_subtype_changed_dm_.end();
@@ -620,30 +687,7 @@ leaf_reporter::report(const class_or_union_diff& d,
 		  represent(*it, ctxt, out, indent + " ",
 			    /*local_only=*/true);
 		  out << "\n";
-		}
-	    }
-	}
-
-      numchanges = d.class_or_union_diff::get_priv()->sorted_changed_dm_.size();
-      num_filtered =
-	d.class_or_union_diff::get_priv()->
-	count_filtered_changed_dm(/*local_only =*/ true);
-      assert(numchanges >= num_filtered);
-      net_numchanges = numchanges - num_filtered;
-
-      if (net_numchanges)
-	{
-	  report_mem_header(out, change_kind, "data member", indent);
-	  for (var_diff_sptrs_type::const_iterator it =
-		 d.class_or_union_diff::get_priv()->sorted_changed_dm_.begin();
-	       it != d.class_or_union_diff::get_priv()->sorted_changed_dm_.end();
-	       ++it)
-	    {
-	      if (diff_to_be_reported((*it).get()))
-		{
-		  represent(*it, ctxt, out, indent + " ",
-			    /*local_only=*/true);
-		  out << "\n";
+		  emitted_data_members_changes = true;
 		}
 	    }
 	}
@@ -834,7 +878,7 @@ leaf_reporter::report(const function_decl_diff& d,
 
   maybe_report_diff_for_symbol(ff->get_symbol(),
 			       sf->get_symbol(),
-			       out, indent);
+			       ctxt, out, indent);
 
   // Now report about inline-ness changes
   if (ff->is_declared_inline() != sf->is_declared_inline())
@@ -946,7 +990,7 @@ leaf_reporter::report(const var_diff& d,
 
   maybe_report_diff_for_symbol(d.first_var()->get_symbol(),
 			       d.second_var()->get_symbol(),
-			       out, indent);
+			       d.context(), out, indent);
 
   maybe_report_diff_for_member(first, second, d.context(), out, indent);
 
@@ -1060,6 +1104,79 @@ leaf_reporter::report(const corpus_diff& d,
 	out << "\n";
     }
 
+  if (ctxt->show_changed_fns())
+    {
+      // Show changed functions.
+      size_t num_changed = s.net_num_leaf_func_changes();
+      if (num_changed == 1)
+	out << indent << "1 function with some sub-type change:\n\n";
+      else if (num_changed > 1)
+	out << indent << num_changed
+	    << " functions with some sub-type change:\n\n";
+
+      bool changed = false;
+      vector<function_decl_diff_sptr> sorted_changed_fns;
+      sort_string_function_decl_diff_sptr_map(d.priv_->changed_fns_map_,
+					      sorted_changed_fns);
+      for (vector<function_decl_diff_sptr>::const_iterator i =
+	     sorted_changed_fns.begin();
+	   i != sorted_changed_fns.end();
+	   ++i)
+	{
+	  diff_sptr diff = *i;
+	  if (!diff)
+	    continue;
+
+	  if (diff_to_be_reported(diff.get()))
+	    {
+	      function_decl_sptr fn = (*i)->first_function_decl();
+	      out << indent << "  [C]'"
+		  << fn->get_pretty_representation() << "'";
+	      report_loc_info((*i)->second_function_decl(), *ctxt, out);
+	      out << " has some sub-type changes:\n";
+	      if ((fn->get_symbol()->has_aliases()
+		   && !(is_member_function(fn)
+			&& get_member_function_is_ctor(fn))
+		   && !(is_member_function(fn)
+			&& get_member_function_is_dtor(fn)))
+		  || (is_c_language(get_translation_unit(fn)->get_language())
+		      && fn->get_name() != fn->get_linkage_name()))
+		{
+		  int number_of_aliases =
+		    fn->get_symbol()->get_number_of_aliases();
+		  if (number_of_aliases == 0)
+		    {
+		      out << indent << "    "
+			  << "Please note that the exported symbol of "
+			"this function is "
+			  << fn->get_symbol()->get_id_string()
+			  << "\n";
+		    }
+		  else
+		    {
+		      out << indent << "    "
+			  << "Please note that the symbol of this function is "
+			  << fn->get_symbol()->get_id_string()
+			  << "\n     and it aliases symbol";
+		      if (number_of_aliases > 1)
+			out << "s";
+		      out << ": "
+			  << fn->get_symbol()->get_aliases_id_string(false)
+			  << "\n";
+		    }
+		}
+	      diff->report(out, indent + "    ");
+	      out << "\n";
+	      changed |= true;
+	    }
+	}
+      if (changed)
+	{
+	  out << "\n";
+	  changed = false;
+	}
+    }
+
   if (ctxt->show_added_fns())
     {
       if (s.net_num_func_added() == 1)
@@ -1113,7 +1230,7 @@ leaf_reporter::report(const corpus_diff& d,
 
   // Now show the changed types.
   const diff_maps& leaf_diffs = d.get_leaf_diffs();
-  report_changes_from_diff_maps(leaf_diffs, out, indent);
+  report_type_changes_from_diff_maps(*this, leaf_diffs, out, indent);
 }
 } // end namespace comparison
 } // end namespace abigail
