@@ -387,6 +387,14 @@ InOutSpec in_out_specs[] =
     "data/test-diff-filter/test30-pr18904-rvalueref-report1.txt",
     "output/test-diff-filter/test30-pr18904-rvalueref-report1.txt",
   },
+  { // Just like the previous test, but emit sizes in hex and bytes
+    "data/test-diff-filter/test30-pr18904-rvalueref-liba.so",
+    "data/test-diff-filter/test30-pr18904-rvalueref-libb.so",
+    "--no-default-suppression --no-linkage-name --no-redundant "
+    "--show-hex --show-bytes",
+    "data/test-diff-filter/test30-pr18904-rvalueref-report2.txt",
+    "output/test-diff-filter/test30-pr18904-rvalueref-report2.txt",
+  },
   {
     "data/test-diff-filter/test31-pr18535-libstdc++-4.8.3.so",
     "data/test-diff-filter/test31-pr18535-libstdc++-4.9.2.so",
@@ -485,6 +493,27 @@ InOutSpec in_out_specs[] =
     "data/test-diff-filter/test42-leaf-report-output-0.txt",
     "output/test-diff-filter/test42-leaf-report-output-0.txt",
   },
+  {
+    "data/test-diff-filter/libtest43-decl-only-def-change-leaf-report-v0.so",
+    "data/test-diff-filter/libtest43-decl-only-def-change-leaf-report-v1.so",
+    "--no-default-suppression --leaf-changes-only",
+    "data/test-diff-filter/test43-decl-only-def-change-leaf-report-0.txt",
+    "output/test-diff-filter/test43-decl-only-def-change-leaf-report-0.txt",
+  },
+  {
+    "data/test-diff-filter/libtest44-anonymous-data-member-v0.so",
+    "data/test-diff-filter/libtest44-anonymous-data-member-v1.so",
+    "--no-default-suppression",
+    "data/test-diff-filter/test44-anonymous-data-member-report-0.txt",
+    "output/test-diff-filter/test44-anonymous-data-member-report-0.txt",
+  },
+  {
+    "data/test-diff-filter/libtest44-anonymous-data-member-v0.so",
+    "data/test-diff-filter/libtest44-anonymous-data-member-v1.so",
+    "--no-default-suppression --leaf-changes-only",
+    "data/test-diff-filter/test44-anonymous-data-member-report-1.txt",
+    "output/test-diff-filter/test44-anonymous-data-member-report-1.txt",
+  },
   // This should be the last entry
   {NULL, NULL, NULL, NULL, NULL}
 };
